#!C:\bld\reprozip-jupyter_1526430084550\_b_env\python.exe
# EASY-INSTALL-ENTRY-SCRIPT: 'reprozip-jupyter==0.4','console_scripts','reprozip-jupyter'
__requires__ = 'reprozip-jupyter==0.4'
import re
import sys
from pkg_resources import load_entry_point

if __name__ == '__main__':
    sys.argv[0] = re.sub(r'(-script\.pyw?|\.exe)?$', '', sys.argv[0])
    sys.exit(
        load_entry_point('reprozip-jupyter==0.4', 'console_scripts', 'reprozip-jupyter')()
    )
